#!/opt/cyops-auth/.env/bin/python
""" Copyright start
  Copyright (C) 2008 - 2020 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import os
import sys
import subprocess
import argparse
from time import sleep

if not os.geteuid() == 0:
    sys.exit("You need to be root to perform this command.")

fields_to_show = []
fields_to_show.append({"label" : "Type           :", "field_name" : "type"})
fields_to_show.append({"label" : "Edition        :", "field_name" : "edition"})
fields_to_show.append({"label" : "Customer Name  :", "field_name" : "customer_name"})
fields_to_show.append({"label" : "Role           :", "field_name" : "role"})
fields_to_show.append({"label" : "Total Users    :", "field_name" : "total_users"})
fields_to_show.append({"label" : "Expiry Date    :", "field_name" : "expiring_on"})
fields_to_show.append({"label" : "Remaining Days :", "field_name" : "remaining_days"})
fields_to_show.append({"label" : "Serial no      :", "field_name" : "serial_no"})

#Below fields will be there in license file when user deploy license
fields_to_show.append({"label" : "Days           :", "field_name" : "days"})
fields_to_show.append({"label" : "Total Users    :", "field_name" : "max_users"})

fields_to_skip_for_perpetual_license = ['expiring_on', 'remaining_days', 'days']
fields_to_skip_for_subscription_license = ['max_users', 'days']

sys.path.append("/opt/cyops-auth")
from handlerworkers.license import CSLicenseManager, NODE_HARDWARE_KEY
from utilities.csconstants import *

def show_license_details(license_details, from_file=False):
    for field in fields_to_show:
        if field['field_name'] in license_details.keys():
            if license_details['type'] == PERPETUAL_TYPE and field['field_name'] in fields_to_skip_for_perpetual_license:
                continue
            if from_file and license_details['type'] == SUBSCRIPTION_TYPE and field['field_name'] in fields_to_skip_for_subscription_license:
                continue
            if license_details[field['field_name']] is not None:
                #Special handling is required for field 'role'
                if field['field_name'] == "role" and len(license_details[field['field_name']]) > 0:
                    print(field['label']  + " " +  str(license_details[field['field_name']][0]))
                else:
                    print(field['label']  + " " +  str(license_details[field['field_name']]))
    if from_file:
        print("Device UUID    : " + license_details['hardware_key']) 
    else:
        print("Device UUID    : " + NODE_HARDWARE_KEY)
    return

def show_installed_license_details():
    try:
        if not CSLicenseManager().is_license_installed(NODE_HARDWARE_KEY):
           print("License not found on the system.")
           exit(1)
    except Exception as e:
        print("License not found on the system.")
        exit(1)
    is_valid_license, message = CSLicenseManager().validate_license()
    if not is_valid_license:
        print(message)
        exit(1)
    license_info = CSLicenseManager().get_license_info()
    current_node_details = license_info['nodes'][NODE_HARDWARE_KEY]['details']
    show_license_details(current_node_details)
    return 

def show_license_details_from_license_file(license_file):
    with open(license_file) as license_file:
        license_key = license_file.read()
    is_valid, new_license_claims = CSLicenseManager().verify_license_signature(license_key)
    if not is_valid:
        message = "License signature verification failed."
        print(message)
        exit(1)
    show_license_details(new_license_claims, from_file=True)

def take_arguments():
    print ("1. Get Device UUID")
    print ("2. Deploy License File")
    print ("Press Ctrl+C to exit")

    option = input("Select an option: ")
    if not option:
        print ("Please provide an input")
        take_arguments()
    else :
        try:
            option = int(option)
            if option != 1 and option != 2:
                print(option)
                print("Wrong option selected. Please Try again")
                sys.exit()
        except ValueError as error:
            print("Invalid input argument provided. Please try again\n")
            take_arguments()

    if option == 1:
        print("Device UUID: " + NODE_HARDWARE_KEY)

    elif option == 2:
        while True:
            option = input("Is the license issued for distributed multi-tenancy setup(y/n/yes/no)? ").lower().strip()
            if option == 'y' or option == 'yes':
                is_distributed = True
                break
            elif option == 'n' or option == 'no':
                is_distributed = False
                break
            else:
                print("Please provide valid input(y/n/yes/no)")
        file_path = input("License file path: ")
        deploy_license(file_path, is_distributed)

def is_valid_file(file):
    if not os.path.exists(file):
        raise argparse.ArgumentTypeError("File {0} does not exist".format(file))
    return file

def start():
    parser = argparse.ArgumentParser()
    parser.prog = "csadm license"
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--get-hkey', dest='get_hkey', action='store_true', help=argparse.SUPPRESS)
    group.add_argument('--get-device-uuid', dest='get_hkey', action='store_true',
                       help='Retrieve device uuid for this instance')
    group.add_argument('--deploy-enterprise-license', dest="enterprise_license", help='Deploy enterprise license', type=is_valid_file)
    group.add_argument('--deploy-multi-tenant-license', dest="distributed_license", help='Deploy multi-tenant license', type=is_valid_file)
    group.add_argument('--show-details', nargs='?', action='store', type=str, dest='license_file_path',
                                     help='Show license details', required=False, default=None, const="dummy_value")
    args = parser.parse_args()
    if args.get_hkey:
        print(NODE_HARDWARE_KEY)
        return
    if args.distributed_license:
        deploy_license(args.distributed_license, True)
        return
    if args.enterprise_license:
        deploy_license(args.enterprise_license, False)
        return
    if args.license_file_path is not None:
        if args.license_file_path == "dummy_value":
           #File is not provided as input
           show_installed_license_details()
        else:
            is_valid_file(args.license_file_path)
            show_license_details_from_license_file(args.license_file_path)
        return
    print("Command to either generate device uuid or to a deploy license\n")
    try:
        take_arguments()
    except KeyboardInterrupt:
        print("")


def deploy_license(license_file, is_distributed):
    try:
        print(format_message_for_cli(INITIAL_MSG))
        with open(license_file) as license_file:
            license_key = license_file.read()
            print(format_message_for_cli(VALIDATING_LICENSE_MSG))
            from handlerworkers.fdnclient import validate_license_before_deployment
            is_valid, message, users, expiry_date = validate_license_before_deployment(NODE_HARDWARE_KEY, license_key, is_distributed, is_cli_deploy=True)

            if is_valid:
                deployment_status, message = CSLicenseManager().deploy_license(NODE_HARDWARE_KEY, license_key, users, expiry_date)
                if deployment_status is True:
                    if is_distributed and message == LICENSE_DEPLOYMENT_SUCCESS_MSG:
                        # Wait for postman and agent service to come up.
                        sleep(10)
                    print(format_message_for_cli(AUTH_RESTART_MSG))
                    subprocess.check_output(['sudo', 'systemctl', 'restart', 'cyops-auth.service'],
                                            stderr=subprocess.DEVNULL).decode('utf-8')
            print(format_message_for_cli(message))
    except FileNotFoundError as error:
        print("The file "+license_file+ " not found. Please try again")


if __name__ == "__main__":
    start()
